using System;
using System.IO;
using System.Collections;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;

namespace FlvComposer
{
    //=====================================================================================//
    //                            public partial class MainForm                            //
    //=====================================================================================//
    public partial class MainForm : Form
    {
        private const int m_defaultFrameDuration = 1000;
        private int m_lastTime = 0;

        //=====================================================================================//
        //                                  public MainForm()                                  //
        //=====================================================================================//
        public MainForm()
        {
            InitializeComponent();
        }

        //=====================================================================================//
        //                              private void SortFrames()                              //
        //=====================================================================================//
        private void SortFrames()
        {
            SortedList<int, string> slist = new SortedList<int, string>();
            foreach (ListViewItem item in ui_lvFrames.Items)
            {
                if (item.SubItems.Count > 1)
                {
                    slist.Add(Convert.ToInt32(item.SubItems[1].Text), item.SubItems[0].Text);
                }
            }
            ui_lvFrames.Items.Clear();
            foreach(int key in slist.Keys)
            {
                ui_lvFrames.Items.Add(slist[key]).SubItems.Add(Convert.ToString(key));
            }
        }

        //=====================================================================================//
        //                          private string FindMediaFolder()                           //
        //=====================================================================================//
        private string FindMediaFolder()
        {
            const string mediaFolderName = "Media";
            const int depth = 10;

            string folder = System.IO.Directory.GetCurrentDirectory();
            for (int i = 0; i < depth; ++i)
            {
                string result = folder + "\\" + mediaFolderName;
                if (System.IO.Directory.Exists(result))
                    return result;
                else folder = folder + "\\..";
            }

            return ".";
        }

        //=====================================================================================//
        //                            private void MainForm_Load()                             //
        //=====================================================================================//
        private void MainForm_Load(object sender, EventArgs e)
        {
            //  find media folder
            string mediaFolder = FindMediaFolder();
            //  create preconstructed bitmaps list
            const int bitmaps = 12;
            for (int i = 0; i < bitmaps; ++i)
            {
                string name = mediaFolder + "\\" + ((i < 10) ? "0" : "") + i.ToString() + ".bmp";
                if (System.IO.File.Exists(name))
                {
                    if (ui_lvFrames.Items.Count > 0)
                        m_lastTime += m_defaultFrameDuration;
                    ui_lvFrames.Items.Add(name).SubItems.Add(Convert.ToString(m_lastTime));
                }
            }
        }

        //=====================================================================================//
        //                   private void ui_lvFrames_ItemSelectionChanged()                   //
        //=====================================================================================//
        private void ui_lvFrames_ItemSelectionChanged(object sender, ListViewItemSelectionChangedEventArgs e)
        {
            ui_imgImage.Image = null;

            if (e.IsSelected)
            {
                string fileName = e.Item.Text;
                if (System.IO.File.Exists(fileName))
                {
                    Cursor.Current = Cursors.WaitCursor;
                    FileStream stream = new FileStream(fileName, FileMode.Open, FileAccess.Read, FileShare.Read);
                    ui_imgImage.Image = new Bitmap(stream);
                    Cursor.Current = Cursors.Default;
                }
            }
            //  enable/disable buttons
            ui_btnChange.Enabled = e.IsSelected;
            ui_btnRemove.Enabled = e.IsSelected;
        }

        //=====================================================================================//
        //                          private void ui_btnInsert_Click()                          //
        //=====================================================================================//
        private void ui_btnInsert_Click(object sender, EventArgs e)
        {
            OpenFileDialog ofd = new OpenFileDialog();
            ofd.Filter = "Bitmap files (*.bmp)|*.bmp";

            if (ofd.ShowDialog() == DialogResult.OK)
            {
                if (ui_lvFrames.Items.Count > 0)
                    m_lastTime  += m_defaultFrameDuration;
                //  add item to frame list
                ui_lvFrames.Items.Add(ofd.FileName).SubItems.Add(Convert.ToString(m_lastTime));
            }
        }

        //=====================================================================================//
        //                          private void ui_btnChange_Click()                          //
        //=====================================================================================//
        private void ui_btnChange_Click(object sender, EventArgs e)
        {
            if (ui_lvFrames.SelectedItems.Count > 0)
            {
                ListViewItem item = ui_lvFrames.SelectedItems[0];
                if (item.SubItems.Count > 1)
                {
                    ListViewItem.ListViewSubItem subItem = item.SubItems[1];
                    //  initialize and show dialog
                    SetFrameSizeForm sfsForm = new SetFrameSizeForm();
                    sfsForm.Time = Convert.ToInt32(subItem.Text);
                    if (sfsForm.ShowDialog(this) == DialogResult.OK)
                    {
                        subItem.Text = Convert.ToString(sfsForm.Time);
                        SortFrames();
                    }
                }
            }
        }

        //=====================================================================================//
        //                          private void ui_btnRemove_Click()                          //
        //=====================================================================================//
        private void ui_btnRemove_Click(object sender, EventArgs e)
        {
            if (ui_lvFrames.SelectedItems.Count > 0)
            {
                ListViewItem item = ui_lvFrames.SelectedItems[0];
                ui_lvFrames.Items.Remove(item);
            }
        }

        //=====================================================================================//
        //                     private void ui_btnCreateFlashVideo_Click()                     //
        //=====================================================================================//
        private void ui_btnCreateFlashVideo_Click(object sender, EventArgs e)
        {
            SaveFileDialog sfd = new SaveFileDialog();
            sfd.Filter = "Flash video files (*.flv)|*.flv";

            if (sfd.ShowDialog() == DialogResult.OK)
            {
                Cursor.Current = Cursors.WaitCursor;
                bool isFailed = false;
                using (FlvCompiler compiler = new FlvCompiler(sfd.FileName))
                {
                    foreach (ListViewItem item in ui_lvFrames.Items)
                    {
                        if (item.SubItems.Count > 1)
                        {
                            if (!compiler.AddFrame(item.SubItems[0].Text, Convert.ToInt32(item.SubItems[1].Text)))
                            {
                                isFailed = true;
                                break;
                            }
                        }
                    }
                }
                if (!isFailed)
                {
                    MessageBox.Show("Flash Video has been created successfully!", "Flash Video Composer", MessageBoxButtons.OK, MessageBoxIcon.Information);
                }
                Cursor.Current = Cursors.Default;
            }
        }
    }
}